@@ -21,8 +21,11 @@
  * @Short_description: A base class for actors that display content
  * @Title: TangleActor
  */
- 
-G_DEFINE_ABSTRACT_TYPE(TangleActor, tangle_actor, CLUTTER_TYPE_ACTOR);
+
+static void clutter_scriptable_iface_init(ClutterScriptableIface* iface);
+
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE(TangleActor, tangle_actor, CLUTTER_TYPE_ACTOR,
+                                 G_IMPLEMENT_INTERFACE(CLUTTER_TYPE_SCRIPTABLE, clutter_scriptable_iface_init););
 
 struct _TangleActorPrivate {
 	gfloat transition_move_x;
@@ -32,6 +35,8 @@ struct _TangleActorPrivate {
 	gulong transition_mode;
 	guint transition_duration;
 
+	guint depth_position;
+
 	gfloat picking_scale_x;
 	gfloat picking_scale_y;
 	gfloat picking_scale_center_x;
@@ -64,6 +69,8 @@ struct _TangleActorPrivate {
 	GList* descendant_events;
 	
 	ClutterAnimation* hiding_animation;
+	
+	ClutterScript* script;
 
 	guint transition_scale_children : 1;
 	guint margin_set : 1;
@@ -84,6 +91,7 @@ enum {
 	PROP_TRANSITION_SCALE_Y,
 	PROP_TRANSITION_MODE,
 	PROP_TRANSITION_DURATION,
+	PROP_DEPTH_POSITION,
 	PROP_PICKING_SCALE_X,
 	PROP_PICKING_SCALE_Y,
 	PROP_PICKING_SCALE_CENTER_X,
@@ -128,6 +136,7 @@ static void unset_transition_scale_children(TangleActor* actor);
 static void on_hiding_animation_completed(TangleActor* actor);
 
 static guint signals[LAST_SIGNAL] = { 0 };
+static ClutterScriptableIface* parent_scriptable_iface = NULL;
 
 ClutterAnimation* tangle_actor_animate(TangleActor* actor, gulong mode, guint duration, const gchar* first_property_name, ...) {
 	ClutterAnimation* animation;
@@ -217,7 +226,7 @@ void tangle_actor_hide_animated(TangleActor* actor) {
 	g_return_if_fail(TANGLE_IS_ACTOR(actor));
 	
 	if (CLUTTER_ACTOR_IS_VISIBLE(CLUTTER_ACTOR(actor)) && !TANGLE_ACTOR_IS_HIDING(actor)) {
-		if (TANGLE_ACTOR_IS_PAINTED(actor)) {
+		if (TANGLE_ACTOR_IS_ALLOCATED(actor)) {
 			g_object_get(G_OBJECT(actor), "allocation", &current_allocation, NULL);
 			actor->priv->transition_scale_children = TRUE;
 			animation = animate_transition(actor, current_allocation, NULL);
@@ -243,7 +252,7 @@ void tangle_actor_hide_all_animated(TangleActor* actor) {
 	actor->priv->hide_all = TRUE;
 	
 	if (CLUTTER_ACTOR_IS_VISIBLE(CLUTTER_ACTOR(actor)) && !TANGLE_ACTOR_IS_HIDING(actor)) {
-		if (TANGLE_ACTOR_IS_PAINTED(actor)) {
+		if (TANGLE_ACTOR_IS_ALLOCATED(actor)) {
 			g_object_get(G_OBJECT(actor), "allocation", &current_allocation, NULL);
 			actor->priv->transition_scale_children = TRUE;
 			animation = animate_transition(actor, current_allocation, NULL);
@@ -271,7 +280,7 @@ void tangle_actor_destroy_animated(TangleActor* actor) {
 	actor->priv->destroy = TRUE;
 	
 	if (CLUTTER_ACTOR_IS_VISIBLE(CLUTTER_ACTOR(actor)) && !TANGLE_ACTOR_IS_HIDING(actor)) {
-		if (TANGLE_ACTOR_IS_PAINTED(actor)) {
+		if (TANGLE_ACTOR_IS_ALLOCATED(actor)) {
 			g_object_get(G_OBJECT(actor), "allocation", &current_allocation, NULL);
 			actor->priv->transition_scale_children = TRUE;
 			animation = animate_transition(actor, current_allocation, NULL);
@@ -292,6 +301,42 @@ void tangle_actor_destroy_animated(TangleActor* actor) {
 	}
 }
 
+/**
+ * tangle_actor_get_depth_position:
+ * @actor: a #TangleActor
+ * 
+ * See :depth-position.
+ *
+ * Return value: the depth position of the actor
+ */
+gint tangle_actor_get_depth_position(TangleActor* actor) {
+
+	return actor->priv->depth_position;
+}
+
+/**
+ * tangle_actor_set_depth_position:
+ * @actor: a #TangleActor
+ * @depth_position: a relative stacking position in depth order
+ *
+ * See :depth-position.
+ *
+ * This function sets ClutterActor:depth to zero as a side effect.
+ */
+void tangle_actor_set_depth_position(TangleActor* actor, gint depth_position) {
+	ClutterActor* parent;
+	
+	if (actor->priv->depth_position != depth_position) {
+		actor->priv->depth_position = depth_position;
+		if (clutter_actor_get_depth(CLUTTER_ACTOR(actor)) != 0.0) {
+			clutter_actor_set_depth(CLUTTER_ACTOR(actor), 0.0);
+		} else if ((parent = clutter_actor_get_parent(CLUTTER_ACTOR(actor))) && CLUTTER_IS_CONTAINER(parent)) {
+			clutter_container_sort_depth_order(CLUTTER_CONTAINER(parent));
+		}
+		g_object_notify(G_OBJECT(actor), "depth-position");
+	}
+}
+
 void tangle_actor_get_margin(TangleActor* actor, TangleSpacing* spacing) {
 	*spacing = actor->priv->margin;
 }
@@ -614,6 +659,32 @@ void tangle_actor_handle_descendant_event(TangleActor* actor, TangleEvent* event
 	}
 }
 
+gchar* tangle_actor_lookup_filename(TangleActor* actor, const gchar* filename) {
+	gchar* path = NULL;
+	gchar* dir;
+	
+	if (g_path_is_absolute(filename)) {
+		path = g_strdup(filename);
+	} else if (actor->priv->script) {
+		path = clutter_script_lookup_filename(actor->priv->script, filename);
+	} else {
+		dir = g_get_current_dir();
+		path = g_build_filename(dir, filename, NULL);
+		if (!g_file_test(path, G_FILE_TEST_EXISTS)) {
+			g_free(path);
+			path = NULL;
+		}
+		g_free(dir);
+	}
+	
+	return path;
+}
+
+ClutterScript* tangle_actor_get_script(TangleActor* actor) {
+
+	return actor->priv->script;
+}
+
 TangleSpacing* tangle_spacing_new(gfloat top, gfloat right, gfloat bottom, gfloat left) {
 	TangleSpacing* spacing;
 	
@@ -722,10 +793,11 @@ static void tangle_actor_allocate(ClutterActor* self, const ClutterActorBox* box
 	ClutterAnimation* animation;
 
 	actor = TANGLE_ACTOR(self);
-	if (TANGLE_ACTOR_IS_PAINTED(actor)) {
+	if (TANGLE_ACTOR_IS_ALLOCATED(actor)) {
 		g_object_get(G_OBJECT(self), "allocation", &old_allocation, NULL);
 	} else {
 		old_allocation = NULL;
+		TANGLE_ACTOR_SET_FLAGS(actor, TANGLE_ACTOR_ALLOCATED);
 	}
 	CLUTTER_ACTOR_CLASS(tangle_actor_parent_class)->allocate(self, box, flags);
 	g_object_get(G_OBJECT(self), "allocation", &new_allocation, NULL);
@@ -809,11 +881,15 @@ static void tangle_actor_pick(ClutterActor* clutter_actor, const ClutterColor* c
 
 static void tangle_actor_hide(ClutterActor* clutter_actor) {
 	TangleActor* actor;
+	ClutterActorBox box = { 0.0, 0.0, 0.0, 0.0 };
 	
 	actor = TANGLE_ACTOR(clutter_actor);
 
-	TANGLE_ACTOR_UNSET_FLAGS(actor, TANGLE_ACTOR_PAINTED);
-
+	TANGLE_ACTOR_UNSET_FLAGS(actor, TANGLE_ACTOR_ALLOCATED);
+	/* TODO: dunno, if this is causing mysterious crashes...
+	CLUTTER_ACTOR_CLASS(tangle_actor_parent_class)->allocate(clutter_actor, &box, CLUTTER_ALLOCATION_NONE);
+	*/
+	
 	if (TANGLE_ACTOR_IS_HIDING(actor)) {
 		TANGLE_ACTOR_UNSET_FLAGS(actor, TANGLE_ACTOR_HIDING);
 
@@ -938,7 +1014,7 @@ static ClutterAnimation* tangle_actor_animate_transition(TangleActor* actor, Clu
 							 "transition-scale-y", 0.0,
 							 NULL);
 		} else if ((parent = clutter_actor_get_parent(CLUTTER_ACTOR(actor))) &&
-		           (!TANGLE_IS_ACTOR(parent) || TANGLE_ACTOR_IS_PAINTED(parent))) {
+		           (!TANGLE_IS_ACTOR(parent) || TANGLE_ACTOR_IS_ALLOCATED(parent))) {
 			/* If the parent is not also a new TangleActor (and thus, doing transition animation),
 			 * animate self.
 			 */
@@ -970,7 +1046,7 @@ static void tangle_actor_paint_event_handler(ClutterActor* self, gpointer user_d
 	TangleActor* actor;
 
 	actor = TANGLE_ACTOR(self);
-	TANGLE_ACTOR_SET_FLAGS(actor, TANGLE_ACTOR_PAINTED);
+
 	cogl_translate(actor->priv->transition_move_x, actor->priv->transition_move_y, 0);
 }
 
@@ -1037,6 +1113,9 @@ static void tangle_actor_set_property(GObject* object, guint prop_id, const GVal
 		case PROP_TRANSITION_DURATION:
 			actor->priv->transition_duration = g_value_get_uint(value);
 			break;
+		case PROP_DEPTH_POSITION:
+			tangle_actor_set_depth_position(actor, g_value_get_int(value));
+			break;
 		case PROP_PICKING_SCALE_X:
 			actor->priv->picking_scale_x = g_value_get_float(value);
 			break;
@@ -1113,6 +1192,9 @@ static void tangle_actor_get_property(GObject* object, guint prop_id, GValue* va
 		case PROP_TRANSITION_DURATION:
 			g_value_set_uint(value, actor->priv->transition_duration);
 			break;
+		case PROP_DEPTH_POSITION:
+			g_value_set_int(value, actor->priv->depth_position);
+			break;
 		case PROP_PICKING_SCALE_X:
 			g_value_set_float(value, actor->priv->picking_scale_x);
 			break;
@@ -1169,13 +1251,17 @@ static void tangle_actor_finalize(GObject* object) {
 	
 	actor = TANGLE_ACTOR(object);
 	g_hash_table_destroy(actor->priv->animations);
-	g_signal_handler_disconnect(actor, actor->priv->paint_event_handler_id);
-	g_signal_handler_disconnect(actor, actor->priv->pick_event_handler_id);
 
 	G_OBJECT_CLASS (tangle_actor_parent_class)->finalize (object);
 }
 
 static void tangle_actor_dispose(GObject* object) {
+	TangleActor* actor;
+	
+	actor = TANGLE_ACTOR(object);
+
+	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(actor->priv->script);
+
 	G_OBJECT_CLASS (tangle_actor_parent_class)->dispose (object);
 }
 
@@ -1262,6 +1348,38 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 	                                                  "The animation duration used when transitioning between allocations",
 	                                                  0, G_MAXUINT, 0,
 	                                                  G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleActor:depth-position:
+	 *
+	 * When an actor is in #TangleWidget container, :depth-position
+	 * defines the relative position of the actor in stacking order.
+	 * An actor that has higher :depth-position is drawn in front of
+	 * an actor that has lower :depth-position.
+	 *
+	 * :depth-position is related to ClutterActor:depth, but it does not
+	 * translate the Z coordinate of the actor. It is only used for
+	 * actors that have ClutterActor:depth set to zero (default),
+	 * otherwise it is ignored.
+	 *
+	 * This can interpreted as follows: First, ClutterActor:depth determines
+	 * the depth of (and the Z coordinate of) the actor. If ClutterActor:depth
+	 * is zero, then :depth-position determines the depth of the actor
+	 * related to other actors that has ClutterActor:depth set to zero.
+	 *
+	 * If an actor is not #TangleActor, but some other #ClutterActor
+	 * the :depth-position is considered zero when stacked in #TangleWidget.
+	 *
+	 * Note! This property does not work in other #ClutterContainer implementations
+	 * than #TangleWidget and derivates.
+	 *
+	 * Setting this property resets ClutterActor:depth to zero, as a side effect.
+	 */
+	g_object_class_install_property(gobject_class, PROP_DEPTH_POSITION,
+	                                g_param_spec_int("depth-position",
+	                                                 "Depth position",
+	                                                 "The relative depth position of the actor in stacking order",
+	                                                 G_MININT, G_MAXINT, 0,
+	                                                 G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 	/**
 	 * TangleActor:picking-scale-x:
 	 */
@@ -1426,7 +1544,7 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 						     G_TYPE_NONE, 2,
 						     CLUTTER_TYPE_ACTOR_BOX, CLUTTER_TYPE_ACTOR_BOX);
 
-	g_type_class_add_private (gobject_class, sizeof (TangleActorPrivate));
+	g_type_class_add_private(gobject_class, sizeof(TangleActorPrivate));
 }
 
 static void tangle_actor_init(TangleActor* actor) {
@@ -1443,6 +1561,34 @@ static void tangle_actor_init(TangleActor* actor) {
 	actor->priv->pick_event_handler_id = g_signal_connect(actor, "pick", G_CALLBACK(tangle_actor_pick_event_handler), NULL);
 }
 
+static gboolean tangle_actor_parse_custom_node(ClutterScriptable* scriptable, ClutterScript* script, GValue* value, const gchar* name, JsonNode* node) {
+	gboolean retvalue = FALSE;
+	TangleActor* actor;
+	
+	actor = TANGLE_ACTOR(scriptable);
+	g_assert(!actor->priv->script || actor->priv->script == script);
+
+	if (!actor->priv->script) {
+		/* TODO: This should be weak, should it not? */
+		g_object_ref(script);
+		actor->priv->script = script;
+	}
+
+	if (parent_scriptable_iface->parse_custom_node) {
+		retvalue = parent_scriptable_iface->parse_custom_node(scriptable, script, value, name, node);
+	}
+	
+	return retvalue;
+}
+
+static void clutter_scriptable_iface_init(ClutterScriptableIface* iface) {
+	if (!(parent_scriptable_iface = parent_scriptable_iface = g_type_interface_peek_parent (iface))) {
+		parent_scriptable_iface = g_type_default_interface_peek(CLUTTER_TYPE_SCRIPTABLE);
+	}
+	
+	iface->parse_custom_node = tangle_actor_parse_custom_node;
+}
+
 static void on_transition_animation_completed(ClutterAnimation* animation, gpointer user_data) {
 	TangleVault* vault;
 	TangleActor* actor;
