@@ -27,7 +27,7 @@
  */
 typedef enum {
 	TANGLE_ACTOR_ON_SCREEN = 1 << 0,
-	TANGLE_ACTOR_PAINTED = 1 << 1,
+	TANGLE_ACTOR_ALLOCATED = 1 << 1,
 	TANGLE_ACTOR_HIDING = 1 << 2
 } TangleActorFlags;
 
@@ -67,7 +67,7 @@ GType tangle_actor_get_type (void) G_GNUC_CONST;
 #define TANGLE_ACTOR_UNSET_FLAGS(a,f)  (((TangleActor*)(a))->flags &= ~(f))
 #define TANGLE_ACTOR_IS_ON_SCREEN(a) ((((TangleActor*)(a))->flags & TANGLE_ACTOR_ON_SCREEN) != FALSE)
 #define TANGLE_ACTOR_IS_MAPPED_ON_SCREEN(a) (CLUTTER_ACTOR_IS_MAPPED(a) && ((((TangleActor*)(a))->flags & TANGLE_ACTOR_ON_SCREEN) != FALSE))
-#define TANGLE_ACTOR_IS_PAINTED(a) ((((TangleActor*)(a))->flags & TANGLE_ACTOR_PAINTED) != FALSE)
+#define TANGLE_ACTOR_IS_ALLOCATED(a) ((((TangleActor*)(a))->flags & TANGLE_ACTOR_ALLOCATED) != FALSE)
 #define TANGLE_ACTOR_IS_HIDING(a) ((((TangleActor*)(a))->flags & TANGLE_ACTOR_HIDING) != FALSE)
 
 ClutterAnimation* tangle_actor_animate(TangleActor* actor, gulong mode, guint duration, const gchar* first_property_name, ...);
@@ -79,6 +79,9 @@ void tangle_actor_hide_animated(TangleActor* actor);
 void tangle_actor_hide_all_animated(TangleActor* actor);
 void tangle_actor_destroy_animated(TangleActor* actor);
 
+gint tangle_actor_get_depth_position(TangleActor* actor);
+void tangle_actor_set_depth_position(TangleActor* actor, gint depth_position);
+
 typedef struct {
 	gfloat left;
 	gfloat top;
@@ -115,6 +118,9 @@ void tangle_actor_send_event(TangleActor* actor, TangleEvent* event);
 void tangle_actor_handle_ancestor_event(TangleActor* actor, TangleEvent* event);
 void tangle_actor_handle_descendant_event(TangleActor* actor, TangleEvent* event);
 
+gchar* tangle_actor_lookup_filename(TangleActor* actor, const gchar* filename);
+ClutterScript* tangle_actor_get_script(TangleActor* actor);
+
 TangleSpacing* tangle_spacing_new(gfloat top, gfloat right, gfloat bottom, gfloat left);
 TangleSpacing* tangle_spacing_copy(const TangleSpacing* spacing);
 void tangle_spacing_free(TangleSpacing* spacing);
