@@ -115,16 +115,18 @@ void tangle_style_apply(TangleStyle* style, GObject* object) {
 
 	old_style = TANGLE_STYLE(g_object_get_qdata(object, quark_object_style));
 	if (old_style != style) {
+		g_object_ref(style);
+
 		if (old_style) {
 			/* TODO: only unset properties that are not part of a new style? */
-			tangle_style_foreach_style_property(old_style, unset_style_property, object);
+			tangle_style_unapply(old_style, object);
 		}
+
 		hash_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
 		g_object_set_qdata_full(object, quark_object_style_properties, hash_table, (GDestroyNotify)g_hash_table_unref);
 		g_signal_connect(object, "notify", G_CALLBACK(on_notify), hash_table);
 
 		tangle_style_foreach_style_property(style, set_style_property, object);
-		g_object_ref(style);
 		g_object_set_qdata(object, quark_object_style, style);
 				
 		g_signal_connect(style, "style-changed", G_CALLBACK(on_style_changed), object);
@@ -139,17 +141,18 @@ void tangle_style_unapply(TangleStyle* style, GObject* object) {
 	old_style = TANGLE_STYLE(g_object_get_qdata(object, quark_object_style));
 	if (old_style == style) {
 		tangle_style_foreach_style_property(style, unset_style_property, object);
-		g_object_unref(style);
 		g_object_set_qdata(object, quark_object_style, NULL);
 		
 		hash_table = (GHashTable*)g_object_get_qdata(object, quark_object_style_properties);
 		if (hash_table) {
 			g_object_set_qdata(object, quark_object_style_properties, NULL);
-			g_signal_handlers_disconnect_by_func(object, G_CALLBACK(on_notify), hash_table);
+			printf("* %d\n", g_signal_handlers_disconnect_by_func(object, G_CALLBACK(on_notify), hash_table));
 		}
 		
 		g_signal_handlers_disconnect_by_func(style, G_CALLBACK(on_style_changed), object);
 		g_object_weak_unref(object, disconnect_style_changed_handler, style);
+
+		g_object_unref(style);
 	}
 }
 
@@ -314,8 +317,7 @@ static void set_style_property(const gchar* name, const GValue* value, gpointer
 	    (hash_table = (GHashTable*)g_object_get_qdata(object, quark_object_style_properties))) {
 	    	if (G_VALUE_HOLDS(value, TANGLE_TYPE_ACTION)) {
 			tangle_action_add(TANGLE_ACTION(g_value_get_object(value)), object, name);
-		} else {
-			param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(object), name);
+		} else if (param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(object), name)) {
 			g_value_init(&current_value, G_PARAM_SPEC_VALUE_TYPE(param_spec));
 			g_object_get_property(object, name, &current_value);
 			if (g_param_value_defaults(param_spec, &current_value)) {
@@ -325,6 +327,8 @@ static void set_style_property(const gchar* name, const GValue* value, gpointer
 				g_hash_table_insert(hash_table, g_strdup(name), NULL);
 			}
 			g_value_unset(&current_value);
+		} else {
+			g_warning("%s: object does not have property called '%s'.", G_STRLOC, name);
 		}
 	}
 }
@@ -346,11 +350,13 @@ static void unset_style_property(const gchar* name, const GValue* value, gpointe
 		BLOCK_NOTIFY_HANDLER(object, hash_table);
 		if (G_VALUE_HOLDS(value, TANGLE_TYPE_ACTION)) {
 			tangle_action_remove(TANGLE_ACTION(g_value_get_object(value)), object, name);		
-		} else {
-			param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(object), name);
+		} else if (param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(object), name)) {
+			g_value_init(&current_value, G_PARAM_SPEC_VALUE_TYPE(param_spec));
 			g_param_value_set_default(param_spec, &current_value);
 			set_object_property(object, name, &current_value);
 			g_value_unset(&current_value);
+		} else {
+			g_warning("%s: object does not have property called '%s'.", G_STRLOC, name);
 		}
 		UNBLOCK_NOTIFY_HANDLER(object, hash_table);
 	}
