@@ -18,8 +18,7 @@
 
 static void clutter_scriptable_iface_init(ClutterScriptableIface *iface);
 
-G_DEFINE_TYPE_WITH_CODE(TangleView, tangle_view, TANGLE_TYPE_WIDGET,
-                        G_IMPLEMENT_INTERFACE(CLUTTER_TYPE_SCRIPTABLE, clutter_scriptable_iface_init););
+G_DEFINE_TYPE(TangleView, tangle_view, TANGLE_TYPE_WIDGET);
 
 enum {
 	PROP_0,
@@ -61,7 +60,10 @@ void tangle_view_set_filename(TangleView* view, const gchar* filename) {
 			view->priv->script_id = 0;
 		}
 		g_free(view->priv->filename);
-		view->priv->filename = g_strdup(filename);
+		view->priv->filename = tangle_actor_lookup_filename(TANGLE_ACTOR(view), filename);
+		if (!view->priv->filename) {
+			g_warning("%s: file '%s' not found.", G_STRLOC, filename);
+		}
 		view->priv->loaded = FALSE;
 		g_object_notify(G_OBJECT(view), "filename");
 	}
@@ -72,18 +74,13 @@ const gchar* tangle_view_get_filename(TangleView* view) {
 	return view->priv->filename;
 }
 
-ClutterScript* tangle_view_get_script(TangleView* view) {
-
-	return view->priv->script;
-}
-
 void tangle_view_load_now(TangleView* view) {
 	GError* error = NULL;
 
 	g_return_if_fail(view->priv->filename != NULL);
 
 	if (!view->priv->loaded) {
-		if (!view->priv->script) {
+		if (!view->priv->script && !(view->priv->script = tangle_actor_get_script(TANGLE_ACTOR(view)))) {
 			view->priv->script = clutter_script_new();
 		}
 
@@ -127,7 +124,7 @@ static ClutterAnimation* tangle_view_animate_transition(TangleActor* actor, Clut
 							 "opacity", 0,
 							 NULL);
 		} else if ((parent = clutter_actor_get_parent(CLUTTER_ACTOR(actor))) &&
-		           (!TANGLE_IS_ACTOR(parent) || TANGLE_ACTOR_IS_PAINTED(parent))) {
+		           (!TANGLE_IS_ACTOR(parent) || TANGLE_ACTOR_IS_ALLOCATED(parent))) {
 			g_object_set(G_OBJECT(actor),
 			             "transition-move-x", (new_box->x2 - new_box->x1) / 4,
 				     "transition-move-y", (new_box->y2 - new_box->y1) / 4,
@@ -206,7 +203,6 @@ static void tangle_view_dispose(GObject* object) {
 	TangleView* view;
 	
 	view = TANGLE_VIEW(object);
-	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(view->priv->script);
 
 	G_OBJECT_CLASS(tangle_view_parent_class)->dispose(object);
 }
@@ -251,24 +247,6 @@ static void tangle_view_init(TangleView* view) {
 	view->priv = G_TYPE_INSTANCE_GET_PRIVATE(view, TANGLE_TYPE_VIEW, TangleViewPrivate);
 }
 
-static gboolean tangle_view_parse_custom_node(ClutterScriptable* scriptable, ClutterScript* script, GValue* value, const gchar* name, JsonNode* node) {
-	TangleView* view;
-	
-	view = TANGLE_VIEW(scriptable);
-	g_assert(!view->priv->script || view->priv->script == script);
-
-	if (!view->priv->script) {
-		g_object_ref(script);
-		view->priv->script = script;
-	}
-
-	return FALSE;
-}
-
-static void clutter_scriptable_iface_init(ClutterScriptableIface *iface) {
-	iface->parse_custom_node = tangle_view_parse_custom_node;
-}
-
 static void add_orphans_to_container(TangleView* view) {
 	GList* objects;
 	GList* list;
