@@ -37,7 +37,8 @@ enum {
 	PROP_BACKGROUND_COLOR_SET,
 	PROP_BACKGROUND_ACTOR,
 	PROP_PREFER_BACKGROUND_SIZE,
-	PROP_PREFER_BACKGROUND_SIZE_SET
+	PROP_PREFER_BACKGROUND_SIZE_SET,
+	PROP_PICKABLE
 };
 
 struct _TangleWidgetPrivate {
@@ -54,10 +55,15 @@ struct _TangleWidgetPrivate {
 	guint background_color_set : 1;
 	guint prefer_background_size : 1;
 	guint prefer_background_size_set : 1;
+	guint pickable : 1;
 };
 
 static void tangle_widget_add_impl(TangleWidget* container, ClutterActor* actor, ClutterActor* existing, gboolean before);
 static gboolean tangle_widget_unapply_stylesheet(TangleStylable* stylable, TangleStylesheet* stylesheet);
+static void change_actor_order(TangleWidget* widget, ClutterActor* actor, ClutterActor* sibling, gboolean before);
+static gint compare_actor_depths(gconstpointer a, gconstpointer b);
+static gint compare_actor_if_higher_depth_position(gconstpointer a, gconstpointer b);
+static void force_actor_depth(ClutterActor* actor, ClutterActor* sibling, gboolean before);
 
 ClutterActor* tangle_widget_new() {
 
@@ -129,13 +135,28 @@ void tangle_widget_foreach_type_swapped(TangleWidget* widget, GType type, Tangle
 	}
 }
 
-void tangle_widget_add_before(TangleWidget* widget, ClutterActor* actor, ClutterActor* existing, const gchar* first_property, ...) {
-	tangle_widget_add_impl(widget, actor, existing, TRUE);
+void tangle_widget_add(TangleWidget* widget, ClutterActor* actor, ...) {
+	GList* existing_in_children;
+	
+	existing_in_children = g_list_find_custom(widget->priv->children, actor, compare_actor_if_higher_depth_position);
+	if (existing_in_children) {
+		tangle_widget_add_impl(TANGLE_WIDGET(widget), actor, CLUTTER_ACTOR(existing_in_children->data), TRUE);
+	} else {
+		tangle_widget_add_impl(TANGLE_WIDGET(widget), actor, NULL, FALSE);
+	}
+	
 	/* TODO: properties */
 }
 
-void tangle_widget_add_after(TangleWidget* widget, ClutterActor* actor, ClutterActor* existing, const gchar* first_property, ...) {
+void tangle_widget_add_before(TangleWidget* widget, ClutterActor* actor, ClutterActor* existing, ...) {
+	tangle_widget_add_impl(widget, actor, existing, TRUE);
+
+	/* TODO: properties */
+}
+
+void tangle_widget_add_after(TangleWidget* widget, ClutterActor* actor, ClutterActor* existing, ...) {
 	tangle_widget_add_impl(widget, actor, existing, FALSE);
+
 	/* TODO: properties */
 }
 
@@ -341,11 +362,16 @@ static void paint_actor_if_mapped_and_not_hiding(ClutterActor* actor) {
 }
 
 static void tangle_widget_pick(ClutterActor* actor, const ClutterColor *color) {
+	TangleWidget* widget;
 	GList* child_in_list;
 
-	CLUTTER_ACTOR_CLASS(tangle_widget_parent_class)->pick(actor, color);
+	widget = TANGLE_WIDGET(actor);
 
-	g_list_foreach(TANGLE_WIDGET(actor)->priv->children,(GFunc)paint_actor_if_mapped_and_not_hiding, NULL);
+	if (widget->priv->pickable) {
+		CLUTTER_ACTOR_CLASS(tangle_widget_parent_class)->pick(actor, color);
+	}
+
+	g_list_foreach(widget->priv->children,(GFunc)paint_actor_if_mapped_and_not_hiding, NULL);
 }
 
 static void tangle_widget_get_preferred_width(TangleActor* actor, gfloat for_height, gboolean interacting, gfloat* min_width_p, gfloat* natural_width_p, gfloat* max_width_p) {
@@ -482,7 +508,8 @@ static void tangle_widget_allocate(ClutterActor* actor, const ClutterActorBox* b
 	TangleWidget* widget;
 	ClutterActorBox actor_box;
 	ClutterActorBox old_children_allocation;
-	GList* child;
+	GList* child_in_children;
+	ClutterActor* child;
 
 	widget = TANGLE_WIDGET(actor);
 
@@ -508,8 +535,11 @@ static void tangle_widget_allocate(ClutterActor* actor, const ClutterActorBox* b
 	if (widget->priv->layout) {
 		tangle_layout_allocate(widget->priv->layout, &widget->priv->children_allocation, flags);
 	} else {
-		for (child = widget->priv->children; child; child = child->next) {
-			clutter_actor_allocate_preferred_size(CLUTTER_ACTOR(child->data), flags);
+		for (child_in_children = widget->priv->children; child_in_children; child_in_children = child_in_children->next) {
+			child = CLUTTER_ACTOR(child_in_children->data);
+			if (CLUTTER_ACTOR_IS_VISIBLE(child)) {
+				clutter_actor_allocate_preferred_size(child, flags);
+			}
 		}
 	}
 }
@@ -559,6 +589,9 @@ static void tangle_widget_set_property(GObject* object, guint prop_id, const GVa
 		case PROP_PREFER_BACKGROUND_SIZE_SET:
 			tangle_widget_set_prefer_background_size_set(widget, g_value_get_boolean(value));
 			break;
+		case PROP_PICKABLE:
+			widget->priv->pickable = g_value_get_boolean(value);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 			break;
@@ -589,6 +622,9 @@ static void tangle_widget_get_property(GObject* object, guint prop_id, GValue* v
 		case PROP_PREFER_BACKGROUND_SIZE_SET:
 			g_value_set_boolean(value, widget->priv->prefer_background_size_set);
 			break;
+		case PROP_PICKABLE:
+			g_value_set_boolean(value, widget->priv->pickable);
+			break;
 	        default:
 		        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		        break;
@@ -705,12 +741,29 @@ static void tangle_widget_class_init(TangleWidgetClass* widget_class) {
 	                                "Whether to use the prefer-background-size property",
 	                                FALSE,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleWidget:pickable:
+	 *
+	 * Whether the widget is pickable or not. The picking is always forwarded to children,
+	 * but this flags controls whether the whole widget area is used when picking.
+	 *
+	 * Especially, when a widget is on top of other actors (layered with #TangleStackLayout, for example),
+	 * it should not be pickable.
+	 */
+	g_object_class_install_property(gobject_class, PROP_PICKABLE,
+	                                g_param_spec_boolean("pickable",
+	                                "Pickable",
+	                                "Whether the widget is pickable or not",
+	                                TRUE,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private(gobject_class, sizeof(TangleWidgetPrivate));
 }
 
 static void tangle_widget_init(TangleWidget* widget) {
 	widget->priv = G_TYPE_INSTANCE_GET_PRIVATE(widget, TANGLE_TYPE_WIDGET, TangleWidgetPrivate);
+	
+	widget->priv->pickable = TRUE;
 }
 
 static void tangle_widget_add_impl(TangleWidget* widget, ClutterActor* actor, ClutterActor* existing, gboolean before) {
@@ -721,6 +774,8 @@ static void tangle_widget_add_impl(TangleWidget* widget, ClutterActor* actor, Cl
 	g_return_if_fail(clutter_actor_get_parent(actor) == NULL);
 	g_return_if_fail(existing == NULL || CLUTTER_IS_ACTOR(existing));
 	g_return_if_fail(existing == NULL || clutter_actor_get_parent(existing) == CLUTTER_ACTOR(widget));
+
+	g_object_ref(actor);
 	
 	if (existing) {
 		existing_in_list = g_list_find(widget->priv->children, existing);
@@ -729,18 +784,21 @@ static void tangle_widget_add_impl(TangleWidget* widget, ClutterActor* actor, Cl
 		if (before) {
 			widget->priv->children = g_list_insert_before(widget->priv->children, existing_in_list, actor);
 		} else {
-			if ((existing_in_list = g_list_next(existing_in_list))) {
-				widget->priv->children = g_list_insert_before(widget->priv->children, existing_in_list, actor);
-			} else {
-				widget->priv->children = g_list_append(widget->priv->children, actor);
-			}
+			widget->priv->children = g_list_insert_before(widget->priv->children, g_list_next(existing_in_list), actor);
 		}
+		force_actor_depth(actor, existing, before);
 	} else {
 		if (before) {
+			existing_in_list = g_list_first(widget->priv->children);
 			widget->priv->children = g_list_prepend(widget->priv->children, actor);
 		} else {
+			/* Unfortunately this traverses the list twice. */
+			existing_in_list = g_list_last(widget->priv->children);
 			widget->priv->children = g_list_append(widget->priv->children, actor);
 		}		
+		if (existing_in_list) {
+			force_actor_depth(actor, CLUTTER_ACTOR(existing_in_list->data), before);
+		}
 	}
 
 	clutter_actor_set_parent(actor, CLUTTER_ACTOR(widget));
@@ -748,13 +806,15 @@ static void tangle_widget_add_impl(TangleWidget* widget, ClutterActor* actor, Cl
 		tangle_stylesheet_apply(widget->priv->stylesheet, G_OBJECT(actor));
 	}
 
-	/* TODO: g_signal_emit_by_name(widget, "actor-added", actor); */
-
 	clutter_actor_queue_relayout(CLUTTER_ACTOR(widget));
+
+	g_signal_emit_by_name(widget, "actor-added", actor);
+
+	g_object_unref(actor);
 }
 
-static void tangle_widget_add(ClutterContainer* widget, ClutterActor* actor) {	
-	tangle_widget_add_impl(TANGLE_WIDGET(widget), actor, NULL, FALSE);
+static void tangle_widget_real_add(ClutterContainer* widget, ClutterActor* actor) {	
+	tangle_widget_add(TANGLE_WIDGET(widget), actor, NULL);
 }
 
 static void tangle_widget_remove(ClutterContainer* widget, ClutterActor* actor) {
@@ -764,6 +824,8 @@ static void tangle_widget_remove(ClutterContainer* widget, ClutterActor* actor)
 	g_return_if_fail(CLUTTER_IS_ACTOR(actor));
 	g_return_if_fail(clutter_actor_get_parent(actor) == CLUTTER_ACTOR(widget));
 
+	g_object_ref(actor);
+
 	was_visible = CLUTTER_ACTOR_IS_VISIBLE(actor);
 
 	TANGLE_WIDGET(widget)->priv->children = g_list_remove(TANGLE_WIDGET(widget)->priv->children, actor);
@@ -772,11 +834,13 @@ static void tangle_widget_remove(ClutterContainer* widget, ClutterActor* actor)
 	}
 	clutter_actor_unparent(actor);
 	
-	/* TODO: g_signal_emit_by_name(widget, "actor-removed", actor); */
-
 	if (was_visible) {
 		clutter_actor_queue_relayout(CLUTTER_ACTOR(widget));
 	}
+	
+	g_signal_emit_by_name(widget, "actor-removed", actor);
+
+	g_object_unref(actor);
 }
 
 static void tangle_widget_foreach(ClutterContainer* widget, ClutterCallback callback, gpointer user_data) {
@@ -789,10 +853,33 @@ static void tangle_widget_foreach(ClutterContainer* widget, ClutterCallback call
 	}
 }
 
+static void tangle_widget_raise(ClutterContainer* container, ClutterActor* actor, ClutterActor* sibling) {
+	change_actor_order(TANGLE_WIDGET(container), actor, sibling, FALSE);
+}
+
+static void tangle_widget_lower(ClutterContainer* container, ClutterActor* actor, ClutterActor* sibling) {
+	change_actor_order(TANGLE_WIDGET(container), actor, sibling, TRUE);
+}
+
+static void tangle_widget_sort_depth_order(ClutterContainer* container) {
+	TangleWidget* widget;
+	
+	widget = TANGLE_WIDGET(container);
+	
+	widget->priv->children = g_list_sort(widget->priv->children, compare_actor_depths);
+	
+	if (CLUTTER_ACTOR_IS_VISIBLE(CLUTTER_ACTOR(widget))) {
+		clutter_actor_queue_redraw(CLUTTER_ACTOR(widget));
+	}
+}
+
 static void clutter_container_iface_init(ClutterContainerIface* iface) {
-	iface->add = tangle_widget_add;
+	iface->add = tangle_widget_real_add;
 	iface->remove = tangle_widget_remove;
 	iface->foreach = tangle_widget_foreach;
+	iface->raise = tangle_widget_raise;
+	iface->lower = tangle_widget_lower;
+	iface->sort_depth_order = tangle_widget_sort_depth_order;
 }
 
 static gboolean tangle_widget_apply_stylesheet(TangleStylable* stylable, TangleStylesheet* stylesheet) {
@@ -827,3 +914,114 @@ static void tangle_stylable_iface_init(TangleStylableIface* iface) {
 	iface->apply_stylesheet = tangle_widget_apply_stylesheet;
 	iface->unapply_stylesheet = tangle_widget_unapply_stylesheet;
 }
+
+static void change_actor_order(TangleWidget* widget, ClutterActor* actor, ClutterActor* sibling, gboolean before) {
+	GList* actor_in_children;
+	GList* sibling_in_children = NULL;
+	
+	actor_in_children = g_list_find(widget->priv->children, actor);
+	g_return_if_fail(actor_in_children != NULL);
+
+	if (sibling) {
+		sibling_in_children = g_list_find(widget->priv->children, sibling);
+		g_return_if_fail(sibling_in_children != NULL);
+	}
+
+	widget->priv->children = g_list_delete_link(widget->priv->children, actor_in_children);
+	
+	if (sibling) {
+		if (before) {
+			widget->priv->children = g_list_insert_before(widget->priv->children, sibling_in_children, actor);
+		} else {
+			widget->priv->children = g_list_insert_before(widget->priv->children, g_list_next(sibling_in_children), actor);
+		}
+		force_actor_depth(actor, sibling, before);
+	} else {
+		if (before) {
+			sibling_in_children = g_list_first(widget->priv->children);
+			widget->priv->children = g_list_prepend(widget->priv->children, actor);
+		} else {
+			/* Unfortunately this traverses the list twice. */
+			sibling_in_children = g_list_last(widget->priv->children);
+			widget->priv->children = g_list_append(widget->priv->children, actor);
+		}
+		if (sibling_in_children) {
+			force_actor_depth(actor, CLUTTER_ACTOR(sibling_in_children->data), before);
+		}
+	}
+}
+
+static gint compare_actor_depths(gconstpointer a, gconstpointer b) {
+	gint retvalue;
+	gfloat depth_a;
+	gfloat depth_b;
+	gint depth_position_a = 0;
+	gint depth_position_b = 0;
+	
+	depth_a = clutter_actor_get_depth(CLUTTER_ACTOR(a));
+	depth_b = clutter_actor_get_depth(CLUTTER_ACTOR(b));
+	if (depth_a == 0.0 && depth_b == 0.0) {
+		if (TANGLE_IS_ACTOR(a)) {
+			depth_position_a = tangle_actor_get_depth_position(TANGLE_ACTOR(a));
+		}
+		if (TANGLE_IS_ACTOR(b)) {
+			depth_position_b = tangle_actor_get_depth_position(TANGLE_ACTOR(b));
+		}
+		retvalue = (depth_position_a < depth_position_b ? -1 : (depth_position_b < depth_position_a ? 1 : 0));
+	} else {
+		retvalue = (depth_a < depth_b ? -1 : (depth_b < depth_a ? 1 : 0));
+	}
+	
+	return retvalue;
+}
+
+static gint compare_actor_if_higher_depth_position(gconstpointer a, gconstpointer b) {
+	gint retvalue;
+	gfloat depth_a;
+	gfloat depth_b;
+	gint depth_position_a = 0;
+	gint depth_position_b = 0;
+	
+	depth_a = clutter_actor_get_depth(CLUTTER_ACTOR(a));
+	depth_b = clutter_actor_get_depth(CLUTTER_ACTOR(b));
+	if (depth_a == 0.0 && depth_b == 0.0) {
+		if (TANGLE_IS_ACTOR(a)) {
+			depth_position_a = tangle_actor_get_depth_position(TANGLE_ACTOR(a));
+		}
+		if (TANGLE_IS_ACTOR(b)) {
+			depth_position_b = tangle_actor_get_depth_position(TANGLE_ACTOR(b));
+		}
+		retvalue = (depth_position_a > depth_position_b ? 0 : -1);
+	} else {
+		retvalue = (depth_a > depth_b ? 0 : -1);
+	}
+
+	return retvalue;
+}
+
+static void force_actor_depth(ClutterActor* actor, ClutterActor* sibling, gboolean before) {
+	gfloat depth_actor;
+	gfloat depth_sibling;
+	gint depth_position_actor = 0;
+	gint depth_position_sibling = 0;
+
+	depth_actor = clutter_actor_get_depth(actor);
+	depth_sibling = clutter_actor_get_depth(sibling);
+	if (depth_actor == 0.0 && depth_sibling == 0.0) {
+		if (TANGLE_IS_ACTOR(actor)) {
+			depth_position_actor = tangle_actor_get_depth_position(TANGLE_ACTOR(actor));
+		}
+		if (TANGLE_IS_ACTOR(sibling)) {
+			depth_position_sibling = tangle_actor_get_depth_position(TANGLE_ACTOR(sibling));
+		}
+		if ((depth_position_actor > depth_position_sibling && before) ||
+		    (depth_position_actor < depth_position_sibling && !before)) {
+			/* This causes unnecessary clutter_container_sort_depth_order(), but no can do. */
+			tangle_actor_set_depth_position(TANGLE_ACTOR(actor), depth_position_sibling);
+		}
+	} else if ((depth_actor > depth_sibling && before) ||
+	           (depth_actor < depth_sibling && !before)) {
+		/* This causes unnecessary clutter_container_sort_depth_order(), but no can do. */
+		clutter_actor_set_depth(actor, depth_sibling);
+	}
+}
