@@ -13,12 +13,14 @@
 
 #include "tangle-action.h"
 #include "tangle-actor.h"
-#include "tangle-call-function-action.h"
+#include "tangle-binding.h"
 #include "tangle-box-layout.h"
 #include "tangle-button.h"
+#include "tangle-call-function-action.h"
 #include "tangle-compound-style.h"
 #include "tangle-draggable-actor.h"
 #include "tangle-droppable-actor.h"
+#include "tangle-event.h"
 #include "tangle-floating-actor.h"
 #include "tangle-grid.h"
 #include "tangle-layout.h"
@@ -26,7 +28,9 @@
 #include "tangle-linear-gradient.h"
 #include "tangle-misc.h"
 #include "tangle-properties.h"
+#include "tangle-scrolling-actor.h"
 #include "tangle-set-property-action.h"
+#include "tangle-show-actor-action.h"
 #include "tangle-stack-layout.h"
 #include "tangle-stylable.h"
 #include "tangle-style.h"
@@ -34,6 +38,7 @@
 #include "tangle-template.h"
 #include "tangle-vault.h"
 #include "tangle-view.h"
+#include "tangle-warehouse.h"
 #include "tangle-widget.h"
 #include "tangle-wrapper-actor.h"
 
